@@ -1,10 +1,9 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Test cases for the zim.formats module.'''
 
-from __future__ import with_statement
+
 
 
 import tests
@@ -15,9 +14,11 @@ from zim.notebook import Path
 from zim.parsing import link_type
 from zim.templates import Template
 
+from xml.etree.ElementTree import ElementTree, Element
+
 
 if not ElementTreeModule.__name__.endswith('cElementTree'):
-	print 'WARNING: using ElementTree instead of cElementTree'
+	print('WARNING: using ElementTree instead of cElementTree')
 
 
 class TestFormatMixin(object):
@@ -36,7 +37,7 @@ class TestFormatMixin(object):
 
 	def testFormatInfo(self):
 		for key in ('name', 'desc', 'mimetype', 'extension'):
-			self.assertIsInstance(self.format.info[key], basestring,
+			self.assertIsInstance(self.format.info[key], str,
 				msg='Invalid key "%s" in format info' % key)
 
 		for key in ('native', 'import', 'export'):
@@ -61,7 +62,8 @@ class TestFormatMixin(object):
 		# No absolute paths ended up in reference
 		pwd = Dir('.')
 		self.assertFalse(pwd.path in text, 'Absolute path ended up in reference')
-		self.assertFalse(pwd.user_path in text, 'Absolute path ended up in reference')
+		if pwd.user_path is not None:
+			self.assertFalse(pwd.user_path in text, 'Absolute path ended up in reference')
 
 		return text
 
@@ -75,8 +77,9 @@ class TestFormatMixin(object):
 		linker = StubLinker(Dir('tests/data/formats'))
 		dumper = self.format.Dumper(linker=linker)
 		result = ''.join(dumper.dump(reftree))
-		#~ print '\n' + '>'*80 + '\n' + result + '\n' + '<'*80 + '\n'
+		#~ print('\n' + '>'*80 + '\n' + result + '\n' + '<'*80 + '\n')
 		self.assertMultiLineEqual(result, wanted)
+		#import ipdb; ipdb.set_trace()
 		self.assertNoTextMissing(result, reftree)
 
 		# Check that dumper did not modify the tree
@@ -85,7 +88,7 @@ class TestFormatMixin(object):
 		# partial dumper
 		parttree = tests.new_parsetree_from_xml("<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial=\"True\">try these <strong>bold</strong>, <emphasis>italic</emphasis></zim-tree>")
 		result = ''.join(dumper.dump(parttree))
-		#~ print ">>>%s<<<" % result
+		#~ print(">>>%s<<<" % result)
 		self.assertFalse(result.endswith('\n')) # partial should not end with "\n"
 
 		# Parser
@@ -101,7 +104,7 @@ class TestFormatMixin(object):
 				# Quick check that we got back *something*
 			string = ''.join(dumper.dump(result))
 				# now we may have loss of formatting, but text should all be there
-				#~ print '\n' + '>'*80 + '\n' + string + '\n' + '<'*80 + '\n'
+				#~ print('\n' + '>'*80 + '\n' + string + '\n' + '<'*80 + '\n')
 			self.assertNoTextMissing(string, reftree)
 
 	_nonalpha_re = re.compile('\W')
@@ -112,33 +115,42 @@ class TestFormatMixin(object):
 		is preserved.
 		'''
 		# TODO how to handle objects ??
-		assert isinstance(text, basestring)
-		offset = 0
-		for elt in tree._etree.iter():
-			if elt.tag == 'img':
-				elttext = (elt.tail) # img text is optional
-			else:
-				elttext = (elt.text, elt.tail)
+		assert isinstance(text, str)
 
-			for wanted in elttext:
-				if not wanted:
-					continue
+		def check_text(wanted):
+			if not wanted:
+				return
 
-				wanted = self._nonalpha_re.sub(' ', wanted)
-					# Non-alpha chars may be replaced with escapes
-					# so no way to hard test them
+			wanted = self._nonalpha_re.sub(' ', wanted)
+			# Non-alpha chars may be replaced with escapes
+			# so no way to hard test them
 
-				if wanted.isspace():
-					continue
+			if wanted.isspace():
+				return
 
-				for piece in wanted.strip().split():
-					#~ print "| >>%s<< @ offset %i" % (piece, offset)
-					try:
-						start = text.index(piece, offset)
-					except ValueError:
-						self.fail('Could not find text piece "%s" in text after offset %i\n>>>%s<<<' % (piece, offset, text[offset:offset+100]))
-					else:
-						offset = start + len(piece)
+			for piece in wanted.strip().split():
+				# ~ print("| >>%s<< @ offset %i" % (piece, offset))
+				try:
+					start = text.index(piece, self.offset)
+				except ValueError:
+					self.fail('Could not find text piece "%s" in text after offset %i\n>>>%s<<<' % (
+						piece, self.offset, text[self.offset:self.offset + 100]))
+				else:
+					self.offset = start + len(piece)
+
+		def loop_tree(tree):  # parse elements one by one, in-depth
+			if type(tree) is Element:
+				if tree.text and tree.tag != "img":  # img text is optional
+					check_text(tree.text)
+
+			for elt in tree.findall("*"):  # if that's a tree or a parent element, dive further
+				loop_tree(elt)
+
+			if type(tree) is Element:
+				check_text(tree.tail)
+
+		self.offset = 0
+		loop_tree(tree._etree)
 
 
 
@@ -200,16 +212,13 @@ class TestParseTree(tests.TestCase):
 		text = tree.tostring()
 		self.assertEqual(text, wanted)
 
-	def testGetHeading(self):
-		'''Test that ParseTree.get_heading() returns the first header's text.
-		'''
+	def testGetHeadingText(self):
 		tree = ParseTree().fromstring(self.xml)
-		self.assertEqual(tree.get_heading(), "Head 1")
+		self.assertEqual(tree.get_heading_text(), "Head 1")
 
-	def testSetHeading(self):
-		'''Test ParseTree.set_heading()'''
+	def testSetHeadingText(self):
 		tree = ParseTree().fromstring(self.xml)
-		tree.set_heading('Foo')
+		tree.set_heading_text('Foo')
 		wanted = '''\
 <?xml version='1.0' encoding='utf-8'?>
 <zim-tree>
@@ -267,12 +276,6 @@ class TestParseTree(tests.TestCase):
 			tree = ParseTree().fromstring(xml)
 			self.assertEqual(tree.get_ends_with_newline(), newline)
 
-	def testGetObjects(self):
-		xml = File('tests/data/formats/parsetree.xml').read().rstrip('\n')
-		tree = tests.new_parsetree_from_xml(xml)
-		objects = list(tree.get_objects())
-		self.assertTrue(len(objects) >= 2)
-
 	def testFindall(self):
 		tree = ParseTree().fromstring(self.xml)
 		wanted = [
@@ -337,18 +340,36 @@ class TestWikiFormat(TestTextFormat):
 
 	def setUp(self):
 		self.format = get_format('wiki')
-		notebook = tests.new_notebook()
+		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 		self.page = notebook.get_page(Path('Foo'))
 
+	def testFormattingBelowHeading(self):
+		input = "====== heading @foo **bold** ======\n"
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><h level="1">heading <tag name="foo">@foo</tag> <strong>bold</strong></h>\n</zim-tree>'''
+		t = self.format.Parser().parse(input)
+		self.assertEqual(t.tostring(), xml)
+		output = self.format.Dumper().dump(t)
+		self.assertEqual(output, input.splitlines(True))
+
+	def testNoNestingBelowVerbatim(self):
+		input = "test 1 2 3 ''code here **not bold!**''\n"
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p>test 1 2 3 <code>code here **not bold!**</code>\n</p></zim-tree>'''
+		t = self.format.Parser().parse(input)
+		self.assertEqual(t.tostring(), xml)
+
 	def testUnicodeBullet(self):
 		'''Test support for unicode bullets in source'''
-		input = u'''\
+		input = '''\
 A list
 • foo
 	• bar
 	• baz
 '''
-		text = u'''\
+		text = '''\
 A list
 * foo
 	* bar
@@ -372,9 +393,9 @@ A list
 			found += 1
 		self.assertEqual(found, 3)
 
-	def testBackward(self):
+	def testBackwardVerbatim(self):
 		'''Test backward compatibility for wiki format'''
-		input = u'''\
+		input = '''\
 test 1 2 3
 
 	Some Verbatim block
@@ -382,7 +403,7 @@ test 1 2 3
 
 test 4 5 6
 '''
-		wanted = u'''\
+		wanted = '''\
 test 1 2 3
 
 \'''
@@ -406,18 +427,31 @@ test 4 5 6
 		output = self.format.Dumper().dump(t)
 		self.assertEqual(output, wanted.splitlines(True))
 
+	def testBackwardURLParsing(self):
+		input = 'Old link: http://///foo.com\n'
+		wanted = 'Old link: [[http://///foo.com]]\n'
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p>Old link: <link href="http://///foo.com">http://///foo.com</link>
+</p></zim-tree>'''
+
+		t = self.format.Parser(version='zim 0.4').parse(input)
+		self.assertEqual(t.tostring(), xml)
+		output = self.format.Dumper().dump(t)
+		self.assertEqual(output, wanted.splitlines(True))
+
 	def testList(self):
 		def check(text, xml, wanted=None):
 			if wanted is None:
 				wanted = text
 
 			tree = self.format.Parser().parse(text)
-			#~ print '>>>\n' + tree.tostring() + '\n<<<'
+			#~ print('>>>\n' + tree.tostring() + '\n<<<')
 			self.assertEqual(tree.tostring(), xml)
 
 			lines = self.format.Dumper().dump(tree)
 			result = ''.join(lines)
-			#~ print '>>>\n' + result + '<<<'
+			#~ print('>>>\n' + result + '<<<')
 			self.assertEqual(result, wanted)
 
 
@@ -468,9 +502,23 @@ C. hmmm
 		check(text, xml)
 
 
-		# Inconsistent lists
-		# ( If first item is number, make all items numbered in sequence
-		#   Otherwise numers will be turned into bullets )
+		# Incosistent list with different checkbox types
+		text = '''\
+* foo
+[ ] bar
+* dus
+'''
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p><ul><li bullet="*">foo</li><li bullet="unchecked-box">bar</li><li bullet="*">dus</li></ul></p></zim-tree>'''
+		wanted = '''\
+* foo
+[ ] bar
+* dus
+'''
+		check(text, xml, wanted)
+
+		# Inconsistent lists get broken in multiple lists
 		text = '''\
 1. foo
 4. bar
@@ -479,12 +527,12 @@ a. dus
 '''
 		xml = '''\
 <?xml version='1.0' encoding='utf-8'?>
-<zim-tree><p><ol start="1"><li>foo</li><li>bar</li><li>hmmm</li><li>dus</li></ol></p></zim-tree>'''
+<zim-tree><p><ol start="1"><li>foo</li><li>bar</li></ol><ul><li bullet="*">hmmm</li></ul><ol start="a"><li>dus</li></ol></p></zim-tree>'''
 		wanted = '''\
 1. foo
 2. bar
-3. hmmm
-4. dus
+* hmmm
+a. dus
 '''
 		check(text, xml, wanted)
 
@@ -496,15 +544,16 @@ a. hmmm
 '''
 		xml = '''\
 <?xml version='1.0' encoding='utf-8'?>
-<zim-tree><p><ul><li bullet="*">foo</li><li bullet="*">bar</li><li bullet="*">hmmm</li><li bullet="*">dus</li></ul></p></zim-tree>'''
+<zim-tree><p><ul><li bullet="*">foo</li></ul><ol start="4"><li>bar</li><li>hmmm</li></ol><ul><li bullet="*">dus</li></ul></p></zim-tree>'''
 		wanted = '''\
 * foo
-* bar
-* hmmm
+4. bar
+5. hmmm
 * dus
 '''
 		check(text, xml, wanted)
 
+
 		# Mixed sub-list
 		text = '''\
 * foo
@@ -578,15 +627,129 @@ hmmm
 '''
 		tree = ParseTree()
 		tree.fromstring(xml)
-		text = ''.join( self.format.Dumper().dump(tree) )
+		text = ''.join(self.format.Dumper().dump(tree))
 		self.assertEqual(text, wanted)
 
+	def testStringEscapeDoesNotGetEvaluated(self):
+		text = "this is not a newline: \\name\n This is not a tab: \\tab \n"
+		tree = self.format.Parser().parse(text)
+		#~ print tree.tostring()
+		output = self.format.Dumper().dump(tree)
+		self.assertEqual(''.join(output), text)
+
+	def testGFMAutolinks(self):
+		text = 'Test 123 www.google.com/search?q=Markup+(business))) 456'
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p>Test 123 <link href="www.google.com/search?q=Markup+(business)">www.google.com/search?q=Markup+(business)</link>)) 456
+</p></zim-tree>'''
+		t = self.format.Parser().parse([text])
+		self.assertEqual(t.tostring(), xml)
+
+	def testMatchingLinkBrackets(self):
+		text = '[[[foo]]] [[[bar[baz]]]'
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p>[<link href="foo">foo</link>] [<link href="bar[baz]">bar[baz]</link>
+</p></zim-tree>'''
+		t = self.format.Parser().parse([text])
+		self.assertEqual(t.tostring(), xml)
+
+	def testNoNestedURLs(self):
+		text = '[[http://example.com|example@example.com]]'
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p><link href="http://example.com">example@example.com</link>
+</p></zim-tree>'''
+		t = self.format.Parser().parse([text])
+		self.assertEqual(t.tostring(), xml)
+
+	def testNoNestedLinks(self):
+		text = '[[http://example.com|[[example@example.com]]]]'
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p><link href="http://example.com">[[example@example.com]]</link>
+</p></zim-tree>'''
+		t = self.format.Parser().parse([text])
+		self.assertEqual(t.tostring(), xml)
+
+
+class TestGFMAutolinks(tests.TestCase):
+	# See https://github.github.com/gfm/#autolinks-extension-
+
+	examples = (
+		# Basic match
+		('www.commonmark.org', True, None),
+		('www.commonmark.org/help', True, None),
+		('http://commonmark.org', True, None),
+		('http://commonmark.org/help', True, None),
+		('commonmark.org', False, None),
+		('commonmark.org/help', False, None),
+
+
+		# No "_" in last two parts domain
+		('www.common_mark.org', False, None),
+		('www.commonmark.org_help', False, None),
+		('www.test_123.commonmark.org', True, None),
+
+		# Trailing punctuation
+		('www.commonmark.org/a.b.', True, '.'),
+		('www.commonmark.org.', True, '.'),
+		('www.commonmark.org?', True, '?'),
+
+		# Trailing ")"
+		('www.google.com/search?q=Markup+(business)', True, None),
+		('www.google.com/search?q=Markup+(business))', True, ')'),
+		('www.google.com/search?q=Markup+(business)))', True, '))'),
+		('www.google.com/search?q=(business))+ok', True, None),
+
+		# Trailing entity reference
+		('www.google.com/search?q=commonmark&hl=en', True, None),
+		('www.google.com/search?q=commonmark&hl;', True, '&hl;'),
+
+		# A "<" always breaks the link
+		('www.commonmark.org/he<lp', True, '<lp'),
+
+		# Email
+		('foo@bar.baz', True, None),
+		('hello@mail+xyz.example', False, None),
+		('hello+xyz@mail.example', True, None),
+		('a.b-c_d@a.b', True, None),
+		('a.b-c_d@a.b.', True, '.'),
+		('a.b-c_d@a.b-', False, None),
+		('a.b-c_d@a.b_', False, None),
+		('@tag', False, None),
+
+		# Examples from bug tracker
+		('https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#container-core-v1-', True, None),
+		('https://da.sharelatex.com/templates/books/springer\'s-monograph-type-svm', True, None),
+		('https://en.wikipedia.org/wiki/80/20_(framing_system)', True, None),
+		('https://bugs.kde.org/buglist.cgi?resolution=---', True, None),
+		('https://vimhelp.org/options.txt.html#\'iskeyword\'', True, None),
+		('https://example.com/foo]', True, None),
+	)
+
+	def testFunctions(self):
+		from zim.formats.wiki import match_url, is_url
+
+		for input, input_is_url, tail in self.examples:
+			if input_is_url:
+				if tail:
+					self.assertEqual(match_url(input), input[:-len(tail)])
+					self.assertFalse(is_url(input))
+				else:
+					self.assertEqual(match_url(input), input)
+					self.assertTrue(is_url(input))
+			else:
+				self.assertEqual(match_url(input), None)
+				self.assertFalse(is_url(input))
+
 
 class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 
 	def setUp(self):
 		self.format = get_format('html')
-		notebook = tests.new_notebook()
+		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 		self.page = notebook.get_page(Path('Foo'))
 
 	def testEncoding(self):
@@ -605,9 +768,9 @@ class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 	def testEmptyLines(self):
 		builder = ParseTreeBuilder()
 		builder.start(FORMATTEDTEXT)
-		builder.append(HEADING, {'level':1}, 'head1')
+		builder.append(HEADING, {'level': 1}, 'head1')
 		builder.text('\n')
-		builder.append(HEADING, {'level':2}, 'head2')
+		builder.append(HEADING, {'level': 2}, 'head2')
 		builder.end(FORMATTEDTEXT)
 		tree = builder.get_parsetree()
 
@@ -686,34 +849,11 @@ class TestRstFormat(tests.TestCase, TestFormatMixin):
 		self.format = get_format('rst')
 
 
-class LatexLoggingFilter(tests.LoggingFilter):
-
-	def __init__(self):
-		tests.LoggingFilter.__init__(
-			self,
-			'zim.formats.latex',
-			('No document type set in template', 'Could not find latex equation')
-		)
-
-
 class TestLatexFormat(tests.TestCase, TestFormatMixin):
 
 	def setUp(self):
 		self.format = get_format('latex')
 
-	def testFormat(self):
-		with LatexLoggingFilter():
-			TestFormatMixin.testFormat(self)
-
-	def testFormatReference(self):
-		# Double check reference did not get broken in updating
-		text = self.getReferenceData()
-
-		# Inlined equation is there
-		self.assertFalse('equation001.png' in text, 'This equation should be inlined')
-		self.assertTrue(r'\begin{math}' in text)
-		self.assertTrue(r'\end{math}' in text)
-
 	def testEncode(self):
 		'''test the escaping of certain characters'''
 		format = get_format('latex')
@@ -725,9 +865,9 @@ class TestLatexFormat(tests.TestCase, TestFormatMixin):
 	def testDocumentType(self):
 		builder = ParseTreeBuilder()
 		builder.start(FORMATTEDTEXT)
-		builder.append(HEADING, {'level':1}, 'head1')
+		builder.append(HEADING, {'level': 1}, 'head1')
 		builder.text('\n')
-		builder.append(HEADING, {'level':2}, 'head2')
+		builder.append(HEADING, {'level': 2}, 'head2')
 		builder.end(FORMATTEDTEXT)
 		tree = builder.get_parsetree()
 
@@ -742,6 +882,26 @@ class TestLatexFormat(tests.TestCase, TestFormatMixin):
 			).dump(tree)
 			self.assertIn(head1, ''.join(lines))
 
+	def testImagesWhitelist(self):
+		builder = ParseTreeBuilder()
+		builder.start(FORMATTEDTEXT)
+		builder.append(IMAGE, {'src': 'test.png'})
+		builder.text('\n')
+		builder.append(IMAGE, {'src': 'test.tiff'})
+		builder.text('\n')
+		builder.append(IMAGE, {'src': 'test.tiff', 'href': 'foo'})
+		builder.text('\n')
+		builder.end(FORMATTEDTEXT)
+		tree = builder.get_parsetree()
+
+		wanted = [
+			'\\includegraphics[]{test.png}\n', '\n',
+			'\\href{test.tiff}{test.tiff}\n', '\n',
+			'\\href{foo}{foo}\n', '\n'
+		]
+		lines = self.format.Dumper(linker=StubLinker()).dump(tree)
+		self.assertEqual(lines, wanted)
+
 
 class StubFile(object):
 
@@ -806,9 +966,8 @@ grrr
    .
 </zim-tree>'''
 
-		# For some reason this does not work with cElementTree.XMLBuilder ...
-		from xml.etree.ElementTree import XMLTreeBuilder
-		builder = XMLTreeBuilder(target=OldParseTreeBuilder())
+		from xml.etree.ElementTree import XMLParser
+		builder = XMLParser(target=OldParseTreeBuilder())
 		builder.feed(input)
 		root = builder.close()
 		tree = ParseTree(root)
@@ -837,4 +996,4 @@ Blaat
 		self.assertEqual(body, 'Blaat\n')
 
 		out = dump_header_lines(meta)
-		self.assertEqual(out+'\nBlaat\n', text)
+		self.assertEqual(out + '\nBlaat\n', text)
